@@ -18,6 +18,7 @@
 #include "atticaprovider_p.h"
 
 #include "question.h"
+#include "tagsfilterchecker.h"
 
 #include <QCollator>
 #include <knewstuffcore_debug.h>
@@ -63,7 +64,7 @@ AtticaProvider::AtticaProvider(const Attica::Provider &provider, const QStringLi
 
 QString AtticaProvider::id() const
 {
-    return m_provider.baseUrl().toString();
+    return m_providerId;
 }
 
 void AtticaProvider::authenticationCredentialsMissing(const KNSCore::Provider &)
@@ -86,9 +87,9 @@ bool AtticaProvider::setProviderXML(const QDomElement &xmldata)
     m_providerManager.addProviderFromXml(doc.toString());
 
     if (!m_providerManager.providers().isEmpty()) {
-        qCDebug(KNEWSTUFFCORE) << "base url of attica provider:" << m_providerManager.providers().last().baseUrl().toString();
+        qCDebug(KNEWSTUFFCORE) << "base url of attica provider:" << m_providerManager.providers().constLast().baseUrl().toString();
     } else {
-        qCritical() << "Could not load provider.";
+        qCCritical(KNEWSTUFFCORE) << "Could not load provider.";
         return false;
     }
     return true;
@@ -105,6 +106,7 @@ void AtticaProvider::providerLoaded(const Attica::Provider &provider)
     qCDebug(KNEWSTUFFCORE) << "Added provider: " << provider.name();
 
     m_provider = provider;
+    m_providerId = provider.baseUrl().toString();
 
     Attica::ListJob<Attica::Category> *job = m_provider.requestCategories();
     connect(job, &BaseJob::finished, this, &AtticaProvider::listOfCategoriesLoaded);
@@ -269,9 +271,29 @@ void AtticaProvider::categoryContentsLoaded(BaseJob *job)
     Content::List contents = listJob->itemList();
 
     EntryInternal::List entries;
+    TagsFilterChecker checker(tagFilter());
+    TagsFilterChecker downloadschecker(downloadTagFilter());
     Q_FOREACH (const Content &content, contents) {
-        mCachedContent.insert(content.id(), content);
-        entries.append(entryFromAtticaContent(content));
+        if (checker.filterAccepts(content.tags())) {
+            bool filterAcceptsDownloads = true;
+            if (content.downloads() > 0) {
+                filterAcceptsDownloads = false;
+                for (const Attica::DownloadDescription &dli : content.downloadUrlDescriptions()) {
+                    if (downloadschecker.filterAccepts(dli.tags())) {
+                        filterAcceptsDownloads = true;
+                        break;
+                    }
+                }
+            }
+            if (filterAcceptsDownloads) {
+                mCachedContent.insert(content.id(), content);
+                entries.append(entryFromAtticaContent(content));
+            } else {
+                qCDebug(KNEWSTUFFCORE) << "Filter has excluded" << content.name() << "on download filter" << downloadTagFilter();
+            }
+        } else {
+            qCDebug(KNEWSTUFFCORE) << "Filter has excluded" << content.name() << "on entry filter" << tagFilter();
+        }
     }
 
     qCDebug(KNEWSTUFFCORE) << "loaded: " << mCurrentRequest.hashForRequest() << " count: " << entries.size();
@@ -415,7 +437,7 @@ bool AtticaProvider::jobSuccess(Attica::BaseJob *job) const
     qCDebug(KNEWSTUFFCORE) << "job error: " << job->metadata().error() << " status code: " << job->metadata().statusCode() << job->metadata().message();
 
     if (job->metadata().error() == Attica::Metadata::NetworkError) {
-        emit signalError(i18n("Network error. (%1)", job->metadata().statusCode()));
+        emit signalError(i18n("Network error %1: %2", job->metadata().statusCode(), job->metadata().error()));
     }
     if (job->metadata().error() == Attica::Metadata::OcsError) {
         if (job->metadata().statusCode() == 200) {
@@ -481,6 +503,7 @@ EntryInternal AtticaProvider::entryFromAtticaContent(const Attica::Content &cont
     entry.setSummary(content.description());
     entry.setShortSummary(content.summary());
     entry.setChangelog(content.changelog());
+    entry.setTags(content.tags());
 
     entry.clearDownloadLinkInformation();
     QList<Attica::DownloadDescription> descs = content.downloadUrlDescriptions();
@@ -493,6 +516,7 @@ EntryInternal AtticaProvider::entryFromAtticaContent(const Attica::Content &cont
         info.id = desc.id();
         info.size = desc.size();
         info.isDownloadtypeLink = desc.type() == Attica::DownloadDescription::LinkDownload;
+        info.tags = desc.tags();
         entry.appendDownloadLinkInformation(info);
     }
 
