@@ -60,33 +60,17 @@ Q_GLOBAL_STATIC(QThreadStorage<EngineProviderLoaderHash>, s_engineProviderLoader
 class EnginePrivate {
 public:
     QList<Provider::CategoryMetadata> categoriesMetadata;
+    Attica::ProviderManager *m_atticaProviderManager = nullptr;
+    QStringList tagFilter;
+    QStringList downloadTagFilter;
 };
 
-// BCI: Add a real d-pointer
-typedef QHash<const Engine *, EnginePrivate *> EnginePrivateHash;
-Q_GLOBAL_STATIC(EnginePrivateHash, d_func)
-static EnginePrivate *d(const Engine* engine)
-{
-    EnginePrivate* ret = d_func()->value(engine);
-    if (!ret) {
-        ret = new EnginePrivate;
-        d_func()->insert(engine, ret);
-    }
-    return ret;
-}
-static void delete_d(const Engine* engine)
-{
-    if (auto d = d_func()) {
-        delete d->take(engine);
-    }
-}
-
 Engine::Engine(QObject *parent)
     : QObject(parent)
     , m_installation(new Installation)
     , m_cache()
     , m_searchTimer(new QTimer)
-    , m_atticaProviderManager(nullptr)
+    , d(new EnginePrivate)
     , m_currentPage(-1)
     , m_pageSize(20)
     , m_numDataJobs(0)
@@ -106,10 +90,10 @@ Engine::~Engine()
     if (m_cache) {
         m_cache->writeRegistry();
     }
-    delete m_atticaProviderManager;
+    delete d->m_atticaProviderManager;
     delete m_searchTimer;
     delete m_installation;
-    delete_d(this);
+    delete d;
 }
 
 bool Engine::init(const QString &configfile)
@@ -121,7 +105,7 @@ bool Engine::init(const QString &configfile)
     KConfig conf(configfile);
     if (conf.accessMode() == KConfig::NoAccess) {
         emit signalError(i18n("Configuration file exists, but cannot be opened: \"%1\"", configfile));
-        qCritical() << "The knsrc file '" << configfile << "' was found but could not be opened.";
+        qCCritical(KNEWSTUFFCORE) << "The knsrc file '" << configfile << "' was found but could not be opened.";
         return false;
     }
 
@@ -134,7 +118,7 @@ bool Engine::init(const QString &configfile)
         group = conf.group("KNewStuff2");
     } else {
         emit signalError(i18n("Configuration file is invalid: \"%1\"", configfile));
-        qCritical() << configfile << " doesn't contain a KNewStuff3 section.";
+        qCCritical(KNEWSTUFFCORE) << configfile << " doesn't contain a KNewStuff3 section.";
         return false;
     }
 
@@ -144,6 +128,12 @@ bool Engine::init(const QString &configfile)
     qCDebug(KNEWSTUFFCORE) << "Categories: " << m_categories;
     m_providerFileUrl = group.readEntry("ProvidersUrl", QString());
 
+    d->tagFilter = group.readEntry("TagFilter", QStringList());
+    if (d->tagFilter.isEmpty()) {
+        d->tagFilter.append(QStringLiteral("ghns_exclude!=1"));
+    }
+    d->downloadTagFilter = group.readEntry("DownloadTagFilter", QStringList());
+
     const QString configFileName = QFileInfo(QDir::isAbsolutePath(configfile) ? configfile : QStandardPaths::locate(QStandardPaths::GenericConfigLocation, configfile)).baseName();
     // let installation read install specific config
     if (!m_installation->readConfig(group)) {
@@ -176,7 +166,7 @@ QStringList Engine::categoriesFilter() const
 
 QList<Provider::CategoryMetadata> Engine::categoriesMetadata()
 {
-    return d(this)->categoriesMetadata;
+    return d->categoriesMetadata;
 }
 
 void Engine::loadProviders()
@@ -184,10 +174,10 @@ void Engine::loadProviders()
     if (m_providerFileUrl.isEmpty()) {
         // it would be nicer to move the attica stuff into its own class
         qCDebug(KNEWSTUFFCORE) << "Using OCS default providers";
-        delete m_atticaProviderManager;
-        m_atticaProviderManager = new Attica::ProviderManager;
-        connect(m_atticaProviderManager, &Attica::ProviderManager::providerAdded, this, &Engine::atticaProviderLoaded);
-        m_atticaProviderManager->loadDefaultProviders();
+        delete d->m_atticaProviderManager;
+        d->m_atticaProviderManager = new Attica::ProviderManager;
+        connect(d->m_atticaProviderManager, &Attica::ProviderManager::providerAdded, this, &Engine::atticaProviderLoaded);
+        d->m_atticaProviderManager->loadDefaultProviders();
     } else {
         qCDebug(KNEWSTUFFCORE) << "loading providers from " << m_providerFileUrl;
         emit signalBusy(i18n("Loading provider information"));
@@ -229,14 +219,14 @@ void Engine::slotProviderFileLoaded(const QDomDocument &doc)
 
         QSharedPointer<KNSCore::Provider> provider;
         if (isAtticaProviderFile || n.attribute(QStringLiteral("type")).toLower() == QLatin1String("rest")) {
-            provider = QSharedPointer<KNSCore::Provider> (new AtticaProvider(m_categories));
+            provider.reset(new AtticaProvider(m_categories));
             connect(provider.data(), &Provider::categoriesMetadataLoded,
                     this, [this](const QList<Provider::CategoryMetadata> &categories){
-                        d(this)->categoriesMetadata = categories;
+                        d->categoriesMetadata = categories;
                         emit signalCategoriesMetadataLoded(categories);
                     });
         } else {
-            provider = QSharedPointer<KNSCore::Provider> (new StaticXmlProvider);
+            provider.reset(new StaticXmlProvider);
         }
 
         if (provider->setProviderXML(n)) {
@@ -260,7 +250,7 @@ void Engine::atticaProviderLoaded(const Attica::Provider &atticaProvider)
         QSharedPointer<KNSCore::Provider> (new AtticaProvider(atticaProvider, m_categories));
     connect(provider.data(), &Provider::categoriesMetadataLoded,
             this, [this](const QList<Provider::CategoryMetadata> &categories){
-                d(this)->categoriesMetadata = categories;
+                d->categoriesMetadata = categories;
                 emit signalCategoriesMetadataLoded(categories);
             });
     addProvider(provider);
@@ -270,6 +260,8 @@ void Engine::addProvider(QSharedPointer<KNSCore::Provider> provider)
 {
     qCDebug(KNEWSTUFFCORE) << "Engine addProvider called with provider with id " << provider->id();
     m_providers.insert(provider->id(), provider);
+    provider->setTagFilter(d->tagFilter);
+    provider->setDownloadTagFilter(d->downloadTagFilter);
     connect(provider.data(), &Provider::providerInitialized, this, &Engine::providerInitialized);
     connect(provider.data(), &Provider::loadingFinished, this, &Engine::slotEntriesLoaded);
     connect(provider.data(), &Provider::entryDetailsLoaded, this, &Engine::slotEntryDetailsLoaded);
@@ -413,6 +405,48 @@ void Engine::setSearchTerm(const QString &searchString)
     }
 }
 
+void Engine::setTagFilter(const QStringList &filter)
+{
+    d->tagFilter = filter;
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+        p->setTagFilter(d->tagFilter);
+    }
+}
+
+QStringList Engine::tagFilter() const
+{
+    return d->tagFilter;
+}
+
+void KNSCore::Engine::addTagFilter(const QString &filter)
+{
+    d->tagFilter << filter;
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+        p->setTagFilter(d->tagFilter);
+    }
+}
+
+void Engine::setDownloadTagFilter(const QStringList &filter)
+{
+    d->downloadTagFilter = filter;
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+        p->setDownloadTagFilter(d->downloadTagFilter);
+    }
+}
+
+QStringList Engine::downloadTagFilter() const
+{
+    return d->downloadTagFilter;
+}
+
+void Engine::addDownloadTagFilter(const QString &filter)
+{
+    d->downloadTagFilter << filter;
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
+        p->setDownloadTagFilter(d->downloadTagFilter);
+    }
+}
+
 void Engine::slotSearchTimerExpired()
 {
     reloadEntries();
