@@ -84,6 +84,10 @@ void DownloadDialog::init(const QString &configFile)
 
     QVBoxLayout *layout = new QVBoxLayout(this);
     d->downloadWidget = new DownloadWidget(configFile, this);
+    // DownloadWidget already provides margins, which would duplicate the dialog layout margins.
+    // As the widget theme could use different margin sizes for a dialog/window outer layout,
+    // so from the duplicated margins we pick the internal margins to remove here.
+    d->downloadWidget->layout()->setMargin(0);
     layout->addWidget(d->downloadWidget);
 
     if (group.hasKey("Name")) {
